/*
 * Carbon Mini Application
 *
 * This is sample application for debugging and learning purposes only.
 */

#include <unistd.h>
#include <Carbon/Carbon.h>

CGContextRef myContext;
WindowRef myWindow;

CGImageRef myImage; 
int myImageHeight, myImageWidth;

MenuRef myMenu;
const MenuCommand kmyCommandPDFExport = FOUR_CHAR_CODE ('Itm1');
const MenuCommand kmyCommandQuit = FOUR_CHAR_CODE ('Itm2');

void InitMyMenu()
{
  MenuItemIndex item;

  CreateNewMenu(1, 0, &myMenu);

  SetMenuTitleWithCFString( myMenu, CFSTR("Sample menu") );
  InsertMenu(myMenu, 0);
  AppendMenuItemTextWithCFString( myMenu, CFSTR("Save as PDF"), 0, 0, &item );
  SetMenuItemCommandID (myMenu, item, kmyCommandPDFExport);
  AppendMenuItemTextWithCFString( myMenu, CFSTR("Quit"), 0, 0, &item );
  SetMenuItemCommandID (myMenu, item, kmyCommandQuit);
}

void InitMyImage()
{
  CFBundleRef myBundle;
  CGDataProviderRef provider; 
  CFURLRef url; 

  myBundle = CFBundleGetMainBundle();
  url = CFBundleCopyResourceURL( myBundle,
  				 CFSTR("ooonologointro"),
				 CFSTR("jpg"),
				 NULL ); 
  provider = CGDataProviderCreateWithURL (url);
  CFRelease (url);
  myImage = CGImageCreateWithJPEGDataProvider (provider, 
					     NULL, 
					     true, 
					     kCGRenderingIntentDefault);
  CGDataProviderRelease (provider);

  myImageWidth = CGImageGetWidth(myImage);
  myImageHeight = CGImageGetHeight(myImage);
  fprintf(stderr, "myImage loaded (%d x %d).\n", myImageWidth, myImageHeight);
}

void DeInitMyImage ()
{
  CGImageRelease (myImage);
}

void MyCreateAndDrawBitmapImage (CGContextRef myFuncContext)
{ 
  CGRect myRect;

  myRect = CGRectMake (0, 0, myImageWidth, myImageHeight);
  CGContextDrawImage (myFuncContext, myRect, myImage);

  CGContextSetRGBFillColor (myFuncContext, 0, 0, 0, 1);

  myRect = CGRectMake (10, 20, 420, 5);
  CGContextFillRect(myFuncContext, myRect);
} 

void MySaveContextIntoPDF ()
{
  //  const char *filename="/tmp/CarbonMiniApp.pdf";
  CFStringRef path;
  CFURLRef url;
  CGContextRef pdfContext;
  CGRect myRect;

  myRect = CGRectMake (0, 0, myImageWidth, myImageHeight);

  path = CFStringCreateWithCString (NULL, "/tmp/CarbonMiniApp.pdf", kCFStringEncodingUTF8);
  url = CFURLCreateWithFileSystemPath (NULL, path, kCFURLPOSIXPathStyle, 0);
  CFRelease (path);

  pdfContext = CGPDFContextCreateWithURL (url, &myRect, NULL);
  CFRelease (url);

  CGContextBeginPage(pdfContext, &myRect);
  MyCreateAndDrawBitmapImage (pdfContext);
  CGContextEndPage (pdfContext);
  CGContextRelease (pdfContext);

  fprintf(stderr, "PDF saved.\n");
}

OSStatus MyWindowEventHandler (EventHandlerCallRef myHandler, EventRef theEvent, void* userData)
{
  OSStatus result = eventNotHandledErr;
  HICommand commandStruct; 

  switch (GetEventKind (theEvent))
    {
    case kEventWindowClose:
 
      fprintf(stderr, "Closing application.\n");
      MySaveContextIntoPDF();
      QuitApplicationEventLoop();
      result = noErr;
      break;
 
    case kEventWindowBoundsChanged:
 
      fprintf(stderr, "Window bounds changed.\n");
      result = noErr;
      break;

    case kEventCommandProcess:
      
      GetEventParameter (theEvent, kEventParamDirectObject, 
			 typeHICommand, NULL, sizeof(HICommand), 
			 NULL, &commandStruct);

      if (commandStruct.commandID == kmyCommandPDFExport)
	MySaveContextIntoPDF();
      else if (commandStruct.commandID == kmyCommandQuit)
	QuitApplicationEventLoop();
      else if (commandStruct.commandID == kHICommandQuit) {
	fprintf(stderr, "Quit!\n");
	QuitApplicationEventLoop();
      }
      else if (commandStruct.commandID == kHICommandHide) {
	fprintf(stderr, "Hide!\n");
      }
      else if (commandStruct.commandID == kHICommandHideOthers) {
	fprintf(stderr, "Hide others!\n");
      }
      else
	fprintf(stderr, "Command: %ld\n", commandStruct.commandID);

      result = noErr;
      break;

    case kEventWindowDrawContent:

      MyCreateAndDrawBitmapImage (myContext);
 
      fprintf(stderr, "Window draw content event happened.\n");
      result = noErr;
      break;
    }

  return (result);
}

void CreateMyWindow() {

  Rect contentBounds;

  CFStringRef windowTitle;
  WindowClass windowClass = kDocumentWindowClass;
  WindowAttributes attributes =
    kWindowStandardDocumentAttributes |
    kWindowStandardHandlerAttribute;

  EventTypeSpec eventList[] = {
    { kEventClassCommand, kEventCommandProcess },
    { kEventClassWindow,  kEventWindowClose },
    { kEventClassWindow,  kEventWindowDrawContent },
    { kEventClassWindow,  kEventWindowBoundsChanged },
  };   

  windowTitle = CFCopyLocalizedStringFromTable(CFSTR("Carbon Mini Application"),
					       CFSTR("CarbonMiniApp"),
					       CFSTR("The title of the application."));

  SetRect(&contentBounds, 100, 100, 100+440, 100+286);

  CreateNewWindow (windowClass,
		   attributes,
		   &contentBounds,
		   &myWindow);
  // FIXME: how to get Graphic Context easily without QuickDraw or getting it from the Event?
  QDBeginCGContext (GetWindowPort (myWindow), &myContext);

  SetWindowTitleWithCFString (myWindow, windowTitle);
  CFRelease(windowTitle);

  InstallWindowEventHandler(myWindow,
			    NewEventHandlerUPP (MyWindowEventHandler),
			    GetEventTypeCount(eventList),
			    eventList, NULL, NULL);

  InitMyMenu();
  ShowWindow(myWindow);
}

int main () {

  InitMyImage();

  CreateMyWindow();
  RunApplicationEventLoop();

  DeInitMyImage();
  return 0;
}
